/* Copyright (C) 2006, 2013  Stephan Kreutzer
 *
 * This file is part of "Random Access File C Example".
 *
 * "Random Access File C Example" is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 or any later version,
 * as published by the Free Software Foundation.
 *
 * "Random Access File C Example" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License 3 for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with "Random Access File C Example". If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * @file $/ra_file.h
 * @brief This module provides the ra_file datatype.
 * @details It is written ANSI-ISO-C++ conform and should be portable without
 *     any kind of problems.
 * @since 2006-11-08
 * @author Stephan Kreutzer
 */



#ifndef _RA_FILE_H
#define _RA_FILE_H



#include <cstdio>



/**
 * @class ra_file
 * @brief This class handles random access file I/O, using C file-I/O.
 */
class ra_file
{
public:
    ra_file();
    ra_file(const ra_file& rhs);
    ~ra_file();

    bool open(const char* path,
              const char* open_mode);
    bool close();
    bool raf_opened();
    long raf_size();
    void* raf_read(void* destination,
                   unsigned long block_offset,
                   unsigned long block_size,
                   unsigned long field_offset,
                   std::size_t n);
    bool raf_write(const void* source,
                   unsigned long block_offset,
                   unsigned long block_size,
                   unsigned long field_offset,
                   std::size_t n);
    bool raf_add(unsigned long block_size);
    bool raf_add(const void* source,
                 unsigned long block_size,
                 unsigned long field_offset,
                 std::size_t n);
private:
    std::FILE* fp;
};



#endif
